// -----------------------------------------------------------------------------
//
// The following code is not required to run the test assuming you comment out the 
// calls to MemoryHook in TestRingSpeed.cpp.
//
// This code is an attempt to monitor all memory allocations while a buffer is being
// tested so it can be reported along with its performance.
//
// Overload "new" and "delete" to track memory usage.
//
// -----------------------------------------------------------------------------

#include <windows.h>

// -----------------------------------------------------------------------------
class MemoryHook 
{
public:
    static long GetByteCnt() { return m_sByteCnt; }
    static long GetMaxByteCnt() { return m_sMaxByteCnt; }

    static void ResetByteCnt(LONG byteCnt = 0, LONG maxByteCnt = 0) 
    {
        m_sByteCnt = byteCnt;  m_sMaxByteCnt = maxByteCnt;
    }

    static volatile LONG m_sByteCnt;
    static volatile LONG m_sMaxByteCnt;
};


// -----------------------------------------------------------------------------
volatile LONG MemoryHook::m_sByteCnt = 0;
volatile LONG MemoryHook::m_sMaxByteCnt = 0;

#define MONITOR_MEMORY
#ifdef MONITOR_MEMORY
// Global new
void* operator new(size_t size)
{
    void* ptr  =  malloc(size);
    InterlockedAdd(&MemoryHook::m_sByteCnt, (LONG) size);
    // MemoryHook::m_sByteCnt += (long)size;
    if (MemoryHook::m_sByteCnt > MemoryHook::m_sMaxByteCnt)
        MemoryHook::m_sMaxByteCnt = MemoryHook::m_sByteCnt;
    return ptr;
}

void* operator new[](size_t size)
{
    void* ptr = malloc(size);
    InterlockedAdd(&MemoryHook::m_sByteCnt, (LONG) size);
    // MemoryHook::m_sByteCnt += (long)size;
    if (MemoryHook::m_sByteCnt > MemoryHook::m_sMaxByteCnt)
        MemoryHook::m_sMaxByteCnt = MemoryHook::m_sByteCnt;
    return ptr;
}

// Global delete
void operator delete(void* ptr)
{
    if (ptr != NULL)
    {
        InterlockedAdd(&MemoryHook::m_sByteCnt, -(LONG) _msize(ptr));
        // MemoryHook::m_sByteCnt -= (long)_msize(ptr);
        free(ptr);
    }
}

void operator delete[](void* ptr)
{
    if (ptr != NULL)
    {
        InterlockedAdd(&MemoryHook::m_sByteCnt, -(LONG) _msize(ptr));
        // MemoryHook::m_sByteCnt -= (long)_msize(ptr);
        free(ptr);
    }
}
#endif

