//  SingeLinkListMSDN.h
//  Wrapper template classon MSDN's Singly Linked List.
//

#pragma once
#include "Lock.h"
#include "IBuffer.h"

// T = data type
// RingSize = ringe size
template <class T, size_t RingSize>
class SList : public IBuffer<T>
{
private:
// we need to align the memory-pool-chunks.
#pragma pack(push, MEMORY_ALLOCATION_ALIGNMENT)

    /// List Item
    template <typename TYPE>
    struct ListItem
    {
        /// Next entry in the single-linked list.
        SLIST_ENTRY Next;
        T value;
    };
    typedef ListItem<T> ListValue;

#pragma pack(pop, MEMORY_ALLOCATION_ALIGNMENT)

    /// Head of the single-linked list.
    SLIST_HEADER m_freeList;
    /// The link-list itself
    ListValue m_freeValues[RingSize];

    /// Head of the single-linked list.
    SLIST_HEADER m_dataList;

    // no copying is supported
    SList& operator=(const SList&);
    SList(const SList&);

public:
    SList()
    {
        InitializeSListHead(&m_freeList);
        for (size_t i = 0; i < RingSize; i++)
        {
            InterlockedPushEntrySList(&m_freeList, &m_freeValues[i].Next);
        }
        InitializeSListHead(&m_dataList);
    }

    ~SList()
    {
        InterlockedFlushSList(&m_freeList);
        InterlockedFlushSList(&m_dataList);
    }

    bool Get(T& value)
    {
        ListValue* pListValue = reinterpret_cast<ListValue*>(InterlockedPopEntrySList(&m_dataList));
        if (NULL == pListValue)
            return false;
        value = pListValue->value;
        InterlockedPushEntrySList(&m_freeList, &pListValue->Next);
        return true;
    }

    bool Put(const T& value)  
    {
        ListValue* pListValue = reinterpret_cast<ListValue*>(InterlockedPopEntrySList(&m_freeList));
        if (NULL == pListValue)
            return false;

        pListValue->value = value;
        InterlockedPushEntrySList(&m_dataList, &pListValue->Next);
        return true;
    }
};