//
//  Code to enable comma format in output stream presentation of numeric values.
//
//  Project: Thread optimized Ring Buffer 
//  Author: Dennis Lang
//  lang.dennis @ comcast.net 
//  http://home.comcast.net/~lang.dennis/

#pragma once
#include <locale>

template <typename Ch>
class numfmt: public std::numpunct<Ch> 
{
  Ch separator; // Character to separate groups
  int group;    // Number of characters in a group
public:
  numfmt(Ch sep, int grp): separator(sep), group(grp) {}
private:
  Ch do_thousands_sep() const { return separator; }
  std::string do_grouping() const { return std::string(1, group); }
};

inline void EnableCommaCout()
{
    char sep = ',';
    int group = 3;
 
    // Imbue both narrow and wide streams for testing
    std::cout.imbue(std::locale(std::locale(),
        new numfmt<char>(sep, group)));
}

inline void DisableCommaCout()
{
    char sep = ',';
    int group = 999;
 
    // Imbue both narrow and wide streams for testing
    std::cout.imbue(std::locale(std::locale(),
        new numfmt<char>(sep, group)));
}

